package estructuras;

/**
 * Interfaz que ser implementada por las clases aplicadoras de algoritmos para
 * construir rboles generadores
 * 
 * @author Daniel Guijarro Enriquez
 * @version 1.0
 */
public interface AplicadorAlgoritmo {

    /**
     * Determina si el paso de elegir vrtice es correcto y, en caso positivo, introduce
     * las modificaciones oportunas en el area de dibujo y actualiza el panel de opciones
     * @param idVertice el vrtice elegido
     * @return true si el paso es correcto; false en caso contrario
     */
    boolean elegirVertice(int idVertice);
    
    /**
     * Determina si el paso de elegir arista es correcto y, en caso positivo, introduce
     * las modificaciones oportunas en el area de dibujo y actualiza el panel de opciones
     * @param idArista la arista elegida
     * @return true si el paso es correcto; false en caso contrario
     */
    boolean elegirArista(int idArista);
    
    /**
     * Determina si el paso de elegir rbol es correcto y, en caso positivo, introduce
     * las modificaciones oportunas en el area de dibujo y actualiza el panel de opciones
     * @return true si el paso es correcto; false en caso contrario
     */
    boolean elegirArbol();
    
    /**
     * Determina si el paso de hallar rbol generador mximo es correcto y,
     * en caso positivo, introduce las modificaciones oportunas en el area de dibujo y
     * actualiza el panel de opciones
     * @return true si el paso es correcto; false en caso contrario
     */
    boolean hallarArbolGeneradorMaximo();
    
    /**
     * Determina si el paso de terminar es correcto y, en caso positivo, introduce
     * las modificaciones oportunas en el area de dibujo y actualiza el panel de opciones
     * @return true si el paso es correcto; false en caso contrario
     */
    boolean terminar();
    
    /**
     * Introduce las modificaciones oportunas en el area de dibujo teniendo en cuenta
     * que en dicho area se desea que quede reflejado el resultado de aplicar un paso
     * correcto del algoritmo.
     * Adems, actualiza el panel de opciones.
     */
    void solucionPasoProximo();
    
    /**
     * Introduce las modificaciones oportunas en el area de dibujo teniendo en cuenta
     * que en dicho area se desea que quede reflejado el resultado de aplicar pasos
     * correctos del algoritmo hasta construir el rbol generador completo.
     * Adems, actualiza el panel de opciones.
     */
    void solucionPasosRestantes();
    
    /**
     * Indica cul es la salida del algoritmo.
     * @return una cadena que indica cul es la salida del algoritmo.
     */
    String salida();
    
}
