package estructuras;

import java.awt.*;

/**
 * Clase de objetos que representan nodos
 * 
 * @author Daniel Guijarro Enriquez
 * @version 1.0
 */
public class Nodo {

    private static final int RADIO = 5;
    private static final Color COLOR_POR_DEFECTO = Color.blue;
    private static final Color COLOR_SELECCIONADO = new Color(191, 191, 0);
    private static final Color COLOR_ARBOL = Color.red;
    private static final Color COLOR_ARBOL_AUX = Color.green;
    private static final Color COLOR_POR_DEFECTO_OSCURO = new Color(0, 0, 0);
    private static final Color COLOR_SELECCIONADO_OSCURO = new Color(127, 127, 0);
    private static final Color COLOR_ARBOL_OSCURO = new Color(127, 0, 0);
    private static final Color COLOR_ARBOL_AUX_OSCURO = new Color(0, 127, 0);
    
    private int x;
    private int y;
    private String etiqueta;
    boolean seleccionado = false;
    boolean perteneceAArbol = false;
    boolean perteneceAArbolAux = false;
    boolean etiquetado = false;
    
    /**
     * Constructor
     * @param coorX coordenada X del nodo a crear
     * @param coorY coordenada Y del nodo a crear
     */
    public Nodo(int coorX, int coorY) {
        x = coorX;
        y = coorY;
    }
    
    /**
     * Devuelve la coordenada X del nodo
     * @return la coordenada X del nodo
     */
    public int getX() {
        return x;
    }
    
    /**
     * Devuelve la coordenada Y del nodo
     * @return la coordenada Y del nodo
     */
    public int getY() {
        return y;
    }
    
    /**
     * Determina si este nodo est seleccionado
     * @return true si este nodo est seleccionado; false en caso contrario
     */
    public boolean estaSeleccionado() {
        return seleccionado;
    }
    
    /**
     * Selecciona o deselecciona este nodo
     * @param seleccionado indica si hay que seleccionar el nodo (true) o deseleccionarlo
     * (false)
     */
    public void setSeleccionado(boolean seleccionado) {
        this.seleccionado = seleccionado;
    }
    
    /**
     * Determina si las coordenadas recibidas como parmetros son del nodo
     * @param coorX coordenada X
     * @param coorY coordenada Y
     * @return true si las coordenadas son del nodo; false en caso contrario
     */
    public boolean esNodoBuscado(int coorX, int coorY) {
        return (coorX >= x - 5) && (coorX <= x + 5) && (coorY >= y - 5) && (coorY <= y + 5);
    }
    
    /**
     * Pinta este nodo
     * @param g contexto grfico para pintar el nodo
     */
    public void pintar(Graphics g) {
        if (seleccionado)
            g.setColor(COLOR_SELECCIONADO);
        else if (perteneceAArbol)
            g.setColor(COLOR_ARBOL);
        else if (perteneceAArbolAux)
            g.setColor(COLOR_ARBOL_AUX);
        else
            g.setColor(COLOR_POR_DEFECTO);
        g.fillOval(x - RADIO, y - RADIO, 2 * RADIO, 2 * RADIO);
    }
    
    /**
     * Pinta la etiqueta de este nodo si ste est etiquetado
     * @param g contexto grfico para pintar la etiqueta
     */
    public void pintarEtiqueta(Graphics g) {
        if (etiquetado) {
            if (seleccionado)
                g.setColor(COLOR_SELECCIONADO_OSCURO);
            else if (perteneceAArbol)
                g.setColor(COLOR_ARBOL_OSCURO);
            else if (perteneceAArbolAux)
                g.setColor(COLOR_ARBOL_AUX_OSCURO);
            else
                g.setColor(COLOR_POR_DEFECTO_OSCURO);
            g.drawString(etiqueta, x - 5, y - 5);
        }
    }
    
    /**
     * Actualiza la posicin de este nodo
     * @param nuevaX coordenada X de la nueva posicin
     * @param nuevaY coordenada Y de la nueva posicin
     */
    public void actualizarPosicin(int nuevaX, int nuevaY) {
        x = nuevaX;
        y = nuevaY;
    }
    
    /**
     * Determina si este nodo pertenece a un rbol generador construido o en construccin
     * @return true en caso positivo; false en caso contrario
     */
    public boolean perteneceAArbol() {
        return perteneceAArbol;
    }
    
    /**
     * Marca o desmarca este nodo como perteneciente a un rbol generador construido o
     * en construccin
     * @param pertenece indica si hay que marcar el nodo (true) o desmarcarlo (false)
     */
    public void setPerteneceAArbol(boolean pertenece) {
        perteneceAArbol = pertenece;
    }
    
    /**
     * Determina si este nodo pertenece a un rbol auxiliar
     * @return true en caso positivo; false en caso contrario
     */
    public boolean perteneceAArbolAux() {
        return perteneceAArbolAux;
    }
    
    /**
     * Marca o desmarca este nodo como perteneciente a un rbol auxiliar
     * @param pertenece indica si hay que marcar el nodo (true) o desmarcarlo (false)
     */
    public void setPerteneceAArbolAux(boolean pertenece) {
        perteneceAArbolAux = pertenece;
    }
    
    /**
     * Etiqueta este nodo
     * @param etiqueta nueva etiqueta para este nodo
     */
    public void etiquetar(String etiqueta) {
        this.etiqueta = etiqueta;
        etiquetado = true;
    }
    
    /**
     * Desetiqueta este nodo
     */
    public void desetiquetar() {
        etiquetado = false;
    }
    
}
