package estructuras;

import java.io.*;
import java.net.*;

public class OperacionesFicheros {

    /**
     * Escribe una cadena de caracteres en un fichero
     * @param cadena la cadena a escribir
     * @param nombreFichero nombre del fichero
     */
    public static void escribir(String cadena, String nombreFichero)
    throws FileNotFoundException {
        PrintWriter fichero = new PrintWriter(nombreFichero);
        fichero.print(cadena);
        fichero.close();
    }
    
    /**
     * Lee el contenido de un fichero y lo devuelve en una cadena de caracteres
     * @param nombreFichero nombre del fichero
     * @return la cadena de caracteres
     */
    public static String leer(String nombreFichero)
    throws FileNotFoundException, IOException {
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);
        BufferedReader fichero = new BufferedReader(new FileReader(nombreFichero));
        String linea;
        while ((linea = fichero.readLine()) != null) {
            pw.println(linea);
        }
        pw.close();
        fichero.close();
        return sw.toString();
    }
    
    /**
     * Lee el contenido de un fichero y lo devuelve en una cadena de caracteres
     * @param urlFichero URL del fichero
     * @return la cadena de caracteres
     */
    public static String leer(URL urlFichero)
    throws IOException {
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);
        InputStream is = urlFichero.openStream();
        BufferedReader br = new BufferedReader(new InputStreamReader(is));
        String linea;
        while ((linea = br.readLine()) != null) {
            pw.println(linea);
        }
        pw.close();
        br.close();
        return sw.toString();
    }
    
}