package estructuras;

import java.util.Random;

/**
 * Clase que representa las preferencias del usuario
 * 
 * @author Daniel Guijarro Enriquez
 * @version 1.0
 */
public class Preferencias {

    private boolean pesosAleatorios = false;
    private int pesoNoAleatorio = 1;
    private int pesoMinimo = 1;
    private int pesoMaximo = 1;
    private Random rnd = new Random();
    
    /**
     * Genera un peso vlido para una nueva arista
     * @return el peso en cuestin
     */
    public int siguientePeso() {
        if (pesosAleatorios)
            return rnd.nextInt(pesoMaximo - pesoMinimo + 1) + pesoMinimo;
        else
            return pesoNoAleatorio;
    }
    
    /**
     * Establece que los pesos que se generen para nuevas aristas deben ser aleatorios y
     * deben estar comprendidos entre un peso mnimo y un peso mximo, ambos incluidos
     * @param pesoMinimo el peso mnimo
     * @param pesoMaximo el peso mximo
     */
    public void setPesoAleatorio(int pesoMinimo, int pesoMaximo) {
        pesosAleatorios = true;
        this.pesoMinimo = pesoMinimo;
        this.pesoMaximo = pesoMaximo;
    }
    
    /**
     * Establece que, para nuevas aristas, se debe generar un peso no aleatorio
     * @param peso el peso en cuestin
     */
    public void setPesoNoAleatorio(int peso) {
        pesosAleatorios = false;
        pesoNoAleatorio = peso;
    }

    /**
     * Indica si los pesos que se generan para nuevas aristas son aleatorios
     * @return true en caso positivo; false en caso contrario
     */
    public boolean sonPesosAleatorios() {
        return pesosAleatorios;
    }

    /**
     * Devuelve el peso mnimo que se puede generar actualmente para una nueva arista
     * si los pesos que se generan para nuevas aristas son aleatorios
     * @return el peso mnimo descrito anteriormente
     */
    public int getPesoMinimo() {
        return pesoMinimo;
    }

    /**
     * Devuelve el peso mximo que se puede generar actualmente para una nueva arista
     * si los pesos que se generan para nuevas aristas son aleatorios
     * @return el peso mximo descrito anteriormente
     */
    public int getPesoMaximo() {
        return pesoMaximo;
    }

    /**
     * Devuelve el peso no aleatorio que se genera actualmente para una nueva arista
     * si los pesos que se generan para nuevas aristas no son aleatorios
     * @return
     */
    public int getPesoNoAleatorio() {
        return pesoNoAleatorio;
    }
    
}
