package interfaz;

import javax.swing.*;

import oyentes.*;
import estructuras.*;

/**
 * Clase que implementa la barra de mens
 * 
 * @author Daniel Guijarro Enriquez
 * @version 1.0
 */
public class BarraMenus extends JMenuBar {

    private static final long serialVersionUID = 1L;
    
    private JMenu archivo = new JMenu("File");
    private JMenuItem abrir = new JMenuItem("Open");
    private JMenuItem cerrar = new JMenuItem("Close");
    private JMenuItem guardar = new JMenuItem("Save");
    private JMenuItem salir = new JMenuItem("Exit");
    
    private JMenu editar = new JMenu("Edit");
    private JMenuItem crearGrafoPredisenyado = new JMenuItem("Create predesigned graph");
    private JMenuItem borrar = new JMenuItem("Delete");
    private JMenuItem cambiarPeso = new JMenuItem("Change weight");
    
    private JMenu algoritmos = new JMenu("Algorithms");
    private JMenu arbolMinimo = new JMenu("Minimum spanning tree");
    private JMenuItem primMinimo = new JMenuItem("Prim algorithm");
    private JMenuItem kruskalMinimo = new JMenuItem("Kruskal algorithm");
    private JMenuItem boruvkaMinimo = new JMenuItem("Borvka algorithm");
    private JMenu arbolMaximo = new JMenu("Maximum spanning tree");
    private JMenuItem primMaximo = new JMenuItem("Prim algorithm");
    private JMenuItem kruskalMaximo = new JMenuItem("Kruskal algorithm");
    private JMenuItem boruvkaMaximo = new JMenuItem("Borvka algorithm");
    private JMenuItem arbolMasUniforme = new JMenuItem("Most uniform spanning tree");
    private JMenuItem arbolMenosUniforme = new JMenuItem("Least uniform spanning tree");
    private JMenuItem caminoDeCapacidadMaxima = new JMenuItem("Maximum capacity path");
    
    private JMenu preferencias = new JMenu("Preferences");
    private JMenuItem pesosParaNuevasAristas = new JMenuItem("Weights for new edges");
    
    /**
     * Constructor
     *
     */
    public BarraMenus() {
        
        // Se aade a la barra de mens el men "Archivo" con todas sus opciones
        archivo.add(abrir);
        archivo.add(cerrar);
        archivo.add(guardar);
        archivo.add(salir);
        add(archivo);
        
        // Se aade a la barra de mens el men "Editar" con todas sus opciones
        editar.add(crearGrafoPredisenyado);
        editar.add(borrar);
        editar.add(cambiarPeso);
        add(editar);
        
        // Se aade a la barra de mens el men "Algoritmos" con todas sus opciones
        arbolMinimo.add(primMinimo);
        arbolMinimo.add(kruskalMinimo);
        arbolMinimo.add(boruvkaMinimo);
        algoritmos.add(arbolMinimo);
        arbolMaximo.add(primMaximo);
        arbolMaximo.add(kruskalMaximo);
        arbolMaximo.add(boruvkaMaximo);
        algoritmos.add(arbolMaximo);
        algoritmos.add(arbolMasUniforme);
        algoritmos.add(arbolMenosUniforme);
        algoritmos.add(caminoDeCapacidadMaxima);
        add(algoritmos);
        
        // Se aade a la barra de mens el men "Preferencias" con todas sus opciones
        preferencias.add(pesosParaNuevasAristas);
        add(preferencias);
        
        // Se desactivan las opciones del men "Editar"
        borrar.setEnabled(false);
        cambiarPeso.setEnabled(false);
    }
    
    /**
     * Registra los oyentes que corresponda en los objetos que componen esta barra de mens
     * @param panelOpciones panel de opciones disponibles durante la aplicacin de un algoritmo
     * @param area rea en la que se permitir dibujar un grafo
     * @param preferencias preferencias del usuario
     * @param ventanaPrincipal ventana principal de la aplicacin
     * @param oyenteRaton objeto oyente de eventos generados al realizar una accin con el ratn
     */
    public void registrarOyentes(PanelOpciones panelOpciones, AreaDibujoGrafo area,
            Preferencias preferencias, VentanaPrincipal ventanaPrincipal,
            OyenteRaton oyenteRaton) {
        abrir.addActionListener(new OyenteAbrir(this, area));
        cerrar.addActionListener(new OyenteCerrar(this, area));
        guardar.addActionListener(new OyenteGuardar(area));
        salir.addActionListener(new OyenteSalir());
        crearGrafoPredisenyado.addActionListener(new OyenteCrearGrafoPredisenyado(
                this, area, ventanaPrincipal));
        borrar.addActionListener(new OyenteBorrar(this, area));
        cambiarPeso.addActionListener(new OyenteCambiarPeso(this, area));
        primMinimo.addActionListener(new OyenteElegirPrimMinimo(this, panelOpciones,
                area, oyenteRaton));
        primMaximo.addActionListener(new OyenteElegirPrimMaximo(this, panelOpciones,
                area, oyenteRaton));
        kruskalMinimo.addActionListener(new OyenteElegirKruskalMinimo(this, panelOpciones,
                area, oyenteRaton));
        kruskalMaximo.addActionListener(new OyenteElegirKruskalMaximo(this, panelOpciones,
                area, oyenteRaton));
        boruvkaMinimo.addActionListener(new OyenteElegirBoruvkaMinimo(this, panelOpciones,
                area, oyenteRaton));
        boruvkaMaximo.addActionListener(new OyenteElegirBoruvkaMaximo(this, panelOpciones,
                area, oyenteRaton));
        arbolMasUniforme.addActionListener(new OyenteElegirMasUniforme(this, panelOpciones,
                area, oyenteRaton));
        arbolMenosUniforme.addActionListener(new OyenteElegirMenosUniforme(this, panelOpciones,
                area, oyenteRaton));
        caminoDeCapacidadMaxima.addActionListener(new OyenteElegirCaminoCapacidadMaxima(
                this, panelOpciones, area, oyenteRaton));
        pesosParaNuevasAristas.addActionListener(new OyentePesosParaNuevasAristas(preferencias,
                ventanaPrincipal));
    }
    
    /**
     * Activa y desactiva las opciones que corresponda del men "Editar" teniendo en cuenta
     * que se deben permitir las correspondientes a operaciones aplicables a una arista
     */
    public void activarOpcionesArista() {
        borrar.setEnabled(true);
        cambiarPeso.setEnabled(true);
    }
    
    /**
     * Activa y desactiva las opciones que corresponda del men "Editar" teniendo en cuenta
     * que se deben permitir las correspondientes a operaciones aplicables a un nodo
     */
    public void activarOpcionesNodo() {
        borrar.setEnabled(true);
        cambiarPeso.setEnabled(false);
    }
    
    /**
     * Desactiva todas las opciones del men "Editar"
     */
    public void desactivarOpcionesNodoYArista() {
        borrar.setEnabled(false);
        cambiarPeso.setEnabled(false);
    }
    
    /**
     * Deja activas slo las opciones que corresponda teniendo en cuenta que se va a aplicar
     * un algoritmo
     */
    public void activarSoloOpcionesAlgoritmo() {
        abrir.setEnabled(false);
        cerrar.setEnabled(false);
        editar.setEnabled(false);
    }
    
    /**
     * Deja activas slo las opciones que corresponda teniendo en cuenta que se va a volver
     * a la edicin del grafo
     */
    public void activarSoloOpcionesEdicionGrafo() {
        abrir.setEnabled(true);
        cerrar.setEnabled(true);
        editar.setEnabled(true);
        borrar.setEnabled(false);
        cambiarPeso.setEnabled(false);
    }
}
