package interfaz;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.util.ArrayList;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.SwingConstants;

import oyentes.*;
import estructuras.*;

public class GestorPanelOpcionesBoruvka {

    private final int NUM_NODOS;
    private final Color FONDO_OPCIONES = new Color(231, 231, 231);  //gris muy claro
    private final ImageIcon FLECHA_VERDE = new ImageIcon(
            this.getClass().getResource("/imgs/icono_flecha_verde.png"));
    private String[] nombresNodos;
    private PanelOpciones panelOpciones;
    private JPanel panelNombreAlgoritmo;
    private JButton volverAEdicionGrafo;
    
    //Panel de componentes (panel que contendr las descripciones de las componentes)
    private JPanel panelComponentes;
    
    private ArrayList<JTextArea> areasTexto;
    private ArrayList<JLabel> etiquetasFlecha;
    private int componenteActual;
    private int numComponentes;
    
    /**
     * Constructor
     * @param panelOpciones panel de opciones disponibles durante la aplicacin de un algoritmo
     * @param area rea de dibujo del grafo
     * @param barra barra de mens
     * @param aplicador aplicador del algoritmo
     * @param numNodos nmero de nodos del grafo
     * @param arbolMinimo indica si se va a aplicar el algoritmo de Borvka
     * para construir el rbol generador mnimo
     */
    public GestorPanelOpcionesBoruvka(PanelOpciones panelOpciones, AreaDibujoGrafo areaDibujo,
            BarraMenus barraMenus, AplicadorAlgoritmo aplicador, int numNodos,
            boolean arbolMinimo) {
        this.panelOpciones = panelOpciones;
        NUM_NODOS = numNodos;
        inicializarNombresNodos();
        JButton construidoArbol;
        if (arbolMinimo) {
            panelNombreAlgoritmo = crearPanelNombreAlgoritmo(
                    "<html><center>Minimum spanning tree:<br></center>" +
                    "<center>Borvka algorithm</center></html>");
            construidoArbol = crearBoton(
                    "<html><center>\"Minimum spanning tree<br></center>" +
                    "<center>already found\"</center></html>", 2);
        }
        else {
            panelNombreAlgoritmo = crearPanelNombreAlgoritmo(
                    "<html><center>Maximum spanning tree:<br></center>" +
                    "<center>Borvka algorithm</center></html>");
            construidoArbol = crearBoton(
                    "<html><center>\"Maximum spanning tree<br></center>" +
                    "<center>already found\"</center></html>", 2);
        }
        construidoArbol.addActionListener(new OyenteTerminar(aplicador));
        JButton solucionPasoProximo = crearBoton("Solution to next step");
        solucionPasoProximo.addActionListener(new OyenteSolucionPasoProximo(aplicador));
        JButton solucionPasosRestantes = crearBoton("Solution to remaining steps");
        solucionPasosRestantes.addActionListener(new OyenteSolucionPasosRestantes(aplicador));
        volverAEdicionGrafo = crearBoton("Back to graph edition");
        volverAEdicionGrafo.addActionListener(new OyenteVolverAEdicionGrafo(barraMenus,
                panelOpciones, areaDibujo));
        JLabel etiqComponentes = crearEtiqComponentes();
        crearPanelComponentes();
        
        panelOpciones.add(panelNombreAlgoritmo);
        panelOpciones.add(construidoArbol);
        panelOpciones.add(solucionPasoProximo);
        panelOpciones.add(solucionPasosRestantes);
        panelOpciones.add(volverAEdicionGrafo);
        panelOpciones.add(etiqComponentes);
        
        panelOpciones.setPreferredSize(new Dimension(260, 283));
        
        panelOpciones.setInferiorVisible(true);
        panelOpciones.setVisible(true);
    }
    
    /**
     * Devuelve el array de nombres de los nodos
     * @return el array de nombres de los nodos
     */
    public String[] getNombresNodos() {
        return nombresNodos;
    }
    
    /**
     * Actualiza el panel de opciones
     * @param listaComponentes lista de las componentes a describir
     */
    public void actualizarPanel(ArrayList<ArrayList<Integer>> listaComponentes) {
        
        //Si se recibe null como parmetro, en el panel de opciones slo se muestra
        //el nombre del algoritmo y la opcin "Volver a edicin del grafo".
        if (listaComponentes == null) {
            panelOpciones.setVisible(false);
            panelOpciones.removeAll();
            panelOpciones.add(panelNombreAlgoritmo);
            panelOpciones.add(volverAEdicionGrafo);
            panelOpciones.setPreferredSize(new Dimension(260, 3000));
            panelOpciones.setVisible(true);
            return;
        }
        
        panelOpciones.setVisible(false);
        
        panelComponentes.removeAll();
        
        areasTexto = new ArrayList<JTextArea>();
        etiquetasFlecha = new ArrayList<JLabel>();
        componenteActual = 0;
        
        int altoPanel = 5;
        JTextArea areaTexto;
        numComponentes = listaComponentes.size();
        for (int i = 0; i < numComponentes; i++) {
            ArrayList<Integer> listaNodos = listaComponentes.get(i);
            int numNodos = listaNodos.size();
            int numLineas = 1 + ((numNodos - 1) / 10);
            int altoEtiqTexto = 3 + (numLineas * 16);
            altoPanel += altoEtiqTexto + 5;
            
            JLabel etiqTexto;
            if (numNodos == 1)
                etiqTexto = new JLabel(nombresNodos[listaNodos.get(0)]);
            else {
                StringBuffer descripcion = new StringBuffer("{");
                for (int j = 0; j < numNodos; j++) {
                    if (j == (numNodos - 1))
                        descripcion.append(nombresNodos[listaNodos.get(j)] + "}");
                    else if (((j + 1) % 10) == 0)
                        descripcion.append(nombresNodos[listaNodos.get(j)] + ",<br>");
                    else
                        descripcion.append(nombresNodos[listaNodos.get(j)] + ", ");
                }
                etiqTexto = new JLabel("<html>" + descripcion.toString() + "</html>");
            }
            etiqTexto.setPreferredSize(new Dimension(313, altoEtiqTexto));
            
            //Se cambia el estilo de la fuente
            Font fuente = etiqTexto.getFont();
            String nombreFuente = fuente.getName();
            int estiloFuente = Font.BOLD;
            int tamaoFuente = fuente.getSize();
            etiqTexto.setFont(new Font(nombreFuente, estiloFuente, tamaoFuente));
            
            if (i == 0)
                areaTexto = new JTextArea(" ");
            else
                areaTexto = new JTextArea();
            areaTexto.setBackground(FONDO_OPCIONES);
            areaTexto.setEditable(false);
            areaTexto.setPreferredSize(new Dimension(6, altoEtiqTexto));
            
            JLabel etiqFlecha;
            if (i == 0)
                etiqFlecha = new JLabel(FLECHA_VERDE);
            else
                etiqFlecha = new JLabel();
            etiqFlecha.setHorizontalAlignment(SwingConstants.LEFT);
            etiqFlecha.setPreferredSize(new Dimension(22, 16));
            
            panelComponentes.add(areaTexto);
            panelComponentes.add(etiqFlecha);
            panelComponentes.add(etiqTexto);
            
            areasTexto.add(areaTexto);
            etiquetasFlecha.add(etiqFlecha);
            
        }
        areaTexto = new JTextArea();
        areaTexto.setBackground(FONDO_OPCIONES);
        areaTexto.setEditable(false);
        areaTexto.setPreferredSize(new Dimension(6, 19));
        altoPanel += 24;
        panelComponentes.add(areaTexto);
        areasTexto.add(areaTexto);
        panelComponentes.setPreferredSize(new Dimension(356, altoPanel));
        
        panelOpciones.setPreferredSize(new Dimension(260, 283));
        
        panelOpciones.setVisible(true);
    }
    
    /**
     * Actualiza el panel de opciones
     *
     */
    public void actualizarPanel() {
        if ((componenteActual + 1) < numComponentes) {
            areasTexto.get(componenteActual + 2).setText(" ");
            etiquetasFlecha.get(componenteActual).setIcon(null);
            etiquetasFlecha.get(componenteActual + 1).setIcon(FLECHA_VERDE);
            componenteActual++;
        }
    }
    
    /**
     * Inicializa la variable nombresNodos (array de nombres de los nodos)
     */
    private void inicializarNombresNodos() {
        nombresNodos = new String[NUM_NODOS];
        int i = 0;
        for (char car1 = ('a' - 1); car1 < ('z' + 1); car1++)
            for (char car2 = 'a'; car2 < ('z' + 1); car2++) {
                if (i == NUM_NODOS)
                    return;
                if (car1 == ('a' - 1))
                    nombresNodos[i] = String.valueOf(car2);
                else
                    nombresNodos[i] = String.valueOf(car1) + car2;
                i++;
            }
    }
    
    /**
     * Crea un panel en el que se indicar qu algoritmo se aplica
     * @param texto texto a mostrar en el panel
     * @return el panel creado
     */
    private JPanel crearPanelNombreAlgoritmo(String texto) {
        JPanel panel = new JPanel();
        panel.setBackground(Color.BLACK);
        panel.setOpaque(true);
        panel.setPreferredSize(new Dimension(260, 70));
        panel.add(crearEtiqNombreAlgoritmo(texto));
        return panel;
    }
    
    /**
     * Crea una etiqueta en la que se indica qu algoritmo se aplica
     * @param texto el texto de la etiqueta
     * @return la etiqueta creada
     */
    private JLabel crearEtiqNombreAlgoritmo(String texto) {
        JLabel etiqueta = new JLabel(texto);
        etiqueta.setForeground(Color.WHITE);
        etiqueta.setHorizontalAlignment(SwingConstants.CENTER);
        etiqueta.setVerticalAlignment(SwingConstants.CENTER);
        
        //Se cambia el tamao de la fuente
        Font fuente = etiqueta.getFont();
        String nombreFuente = fuente.getName();
        int estiloFuente = fuente.getStyle();
        int tamaoFuente = fuente.getSize() + 2;
        etiqueta.setFont(new Font(nombreFuente, estiloFuente, tamaoFuente));
        
        etiqueta.setPreferredSize(new Dimension(250, 60));
        return etiqueta;
    }
    
    /**
     * Crea un botn con una lnea de texto para el panel de opciones
     * @param texto el texto del botn
     * @return el botn creada
     */
    private JButton crearBoton(String texto) {
        JButton boton = new JButton(texto);
        boton.setBorderPainted(true);
        boton.setPreferredSize(new Dimension(255, 25));
        return boton;
    }
    
    /**
     * Crea un botn para el panel de opciones
     * @param texto el texto del botn
     * @param numLineas el nmero de lneas del texto del botn
     * @return el botn creada
     */
    private JButton crearBoton(String texto, int numLineas) {
        JButton boton = new JButton(texto);
        boton.setBorderPainted(true);
        boton.setPreferredSize(new Dimension(255, 8 + (17 * numLineas)));
        return boton;
    }
    
    /**
     * Crea una etiqueta para indicar en el panel de opciones que a continuacin se
     * describen las componentes conexas
     * @return la etiqueta creada
     */
    private JLabel crearEtiqComponentes() {
        JLabel etiqueta = new JLabel(
                "<html>Connected components (the green arrow<br>" +
                "indicates the component for which you can<br>" +
                "choose an edge):<html>");
        etiqueta.setVerticalAlignment(SwingConstants.BOTTOM);
        etiqueta.setPreferredSize(new Dimension(250, 58));
        
        //Se cambia el estilo de la fuente
        Font fuente = etiqueta.getFont();
        String nombreFuente = fuente.getName();
        int estiloFuente = Font.BOLD;
        int tamaoFuente = fuente.getSize();
        etiqueta.setFont(new Font(nombreFuente, estiloFuente, tamaoFuente));
        
        return etiqueta;
    }
    
    /**
     * Crea un panel de componentes
     *
     */
    private void crearPanelComponentes() {
        panelComponentes = panelOpciones.getInterno();
    }
    
}
