package interfaz;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import oyentes.*;

import estructuras.AplicadorAlgoritmo;

public class GestorPanelOpcionesCaminoCapacidadMaxima {

    private PanelOpciones panelOpciones;
    private JPanel panelNombreAlgoritmo;
    private JButton volverAEdicionGrafo;
    
    /**
     * Constructor
     * @param panelOpciones panel de opciones disponibles durante la aplicacin de un algoritmo
     * @param areaDibujo rea de dibujo del grafo
     * @param barraMenus barra de mens
     * @param aplicador aplicador del algoritmo
     */
    public GestorPanelOpcionesCaminoCapacidadMaxima(PanelOpciones panelOpciones,
            AreaDibujoGrafo areaDibujo, BarraMenus barraMenus, AplicadorAlgoritmo aplicador) {
        
        this.panelOpciones = panelOpciones;
        panelOpciones.setPreferredSize(new Dimension(260, 3000));
        
        panelNombreAlgoritmo = crearPanelNombreAlgoritmo("Maximum capacity path");
        JButton construidoArbol = crearBoton(
                "<html><center>\"Maximum capacity path<br></center>" +
                "<center>already found\"</center></html>", 2);
        construidoArbol.addActionListener(new OyenteTerminar(aplicador));
        JButton hallarArbolMinimo = crearBoton("Find minimum spanning tree");
        hallarArbolMinimo.addActionListener(new OyenteHallarOtroArbol());
        JButton hallarArbolMaximo = crearBoton("Find maximum spanning tree");
        hallarArbolMaximo.addActionListener(new OyenteHallarArbolGeneradorMaximo(aplicador));
        JButton hallarArbolMasUniforme = crearBoton("Find most uniform spanning tree");
        hallarArbolMasUniforme.addActionListener(new OyenteHallarOtroArbol());
        JButton hallarArbolMenosUniforme = crearBoton("Find least uniform spanning tree");
        hallarArbolMenosUniforme.addActionListener(new OyenteHallarOtroArbol());
        JButton solucionPasoProximo = crearBoton("Solution to next step");
        solucionPasoProximo.addActionListener(new OyenteSolucionPasoProximo(aplicador));
        JButton solucionPasosRestantes = crearBoton("Solution to remaining steps");
        solucionPasosRestantes.addActionListener(new OyenteSolucionPasosRestantes(aplicador));
        volverAEdicionGrafo = crearBoton("Back to graph edition");
        volverAEdicionGrafo.addActionListener(new OyenteVolverAEdicionGrafo(barraMenus,
                panelOpciones, areaDibujo));
        panelOpciones.add(panelNombreAlgoritmo);
        panelOpciones.add(hallarArbolMinimo);
        panelOpciones.add(hallarArbolMaximo);
        panelOpciones.add(hallarArbolMasUniforme);
        panelOpciones.add(hallarArbolMenosUniforme);
        panelOpciones.add(construidoArbol);
        panelOpciones.add(solucionPasoProximo);
        panelOpciones.add(solucionPasosRestantes);
        panelOpciones.add(volverAEdicionGrafo);
        
        panelOpciones.setVisible(true);
    }
    
    /**
     * Actualiza el panel de opciones
     *
     */
    public void actualizarPanel() {
        panelOpciones.setVisible(false);
        panelOpciones.removeAll();
        panelOpciones.add(panelNombreAlgoritmo);
        panelOpciones.add(volverAEdicionGrafo);
        panelOpciones.setPreferredSize(new Dimension(260, 3000));
        panelOpciones.setVisible(true);
    }
    
    /**
     * Crea un panel en el que se indicar qu algoritmo se aplica
     * @param texto texto a mostrar en el panel
     * @return el panel creado
     */
    private JPanel crearPanelNombreAlgoritmo(String texto) {
        JPanel panel = new JPanel();
        panel.setBackground(Color.BLACK);
        panel.setOpaque(true);
        panel.setPreferredSize(new Dimension(260, 70));
        panel.add(crearEtiqNombreAlgoritmo(texto));
        return panel;
    }
    
    /**
     * Crea una etiqueta en la que se indica qu algoritmo se aplica
     * @param texto el texto de la etiqueta
     * @return la etiqueta creada
     */
    private JLabel crearEtiqNombreAlgoritmo(String texto) {
        JLabel etiqueta = new JLabel(texto);
        etiqueta.setForeground(Color.WHITE);
        etiqueta.setHorizontalAlignment(SwingConstants.CENTER);
        etiqueta.setVerticalAlignment(SwingConstants.CENTER);
        
        //Se cambia el tamao de la fuente
        Font fuente = etiqueta.getFont();
        String nombreFuente = fuente.getName();
        int estiloFuente = fuente.getStyle();
        int tamaoFuente = fuente.getSize() + 2;
        etiqueta.setFont(new Font(nombreFuente, estiloFuente, tamaoFuente));
        
        etiqueta.setPreferredSize(new Dimension(250, 60));
        return etiqueta;
    }
    
    /**
     * Crea un botn con una lnea de texto para el panel de opciones
     * @param texto el texto del botn
     * @return el botn creada
     */
    private JButton crearBoton(String texto) {
        JButton boton = new JButton(texto);
        boton.setBorderPainted(true);
        boton.setPreferredSize(new Dimension(255, 25));
        return boton;
    }
    
    /**
     * Crea un botn para el panel de opciones
     * @param texto el texto del botn
     * @param numLineas el nmero de lneas del texto del botn
     * @return el botn creada
     */
    private JButton crearBoton(String texto, int numLineas) {
        JButton boton = new JButton(texto);
        boton.setBorderPainted(true);
        boton.setPreferredSize(new Dimension(255, 8 + (17 * numLineas)));
        return boton;
    }
    
}
