package interfaz;

import java.util.*;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.SwingConstants;

import oyentes.*;

import estructuras.AplicadorAlgoritmo;

public class GestorPanelOpcionesGalilSchieber {

    private final int NUM_ARISTAS;
    private final Color FONDO_OPCIONES = new Color(231, 231, 231);  //gris muy claro
    private final ImageIcon FLECHA_VERDE = new ImageIcon(
            this.getClass().getResource("/imgs/icono_flecha_verde.png"));
    private Vector<Integer> aristasOrdenadas;
    private boolean arbolMasUniforme;
    private String[] nombresAristas;
    private PanelOpciones panelOpciones;
    private JPanel panelNombreAlgoritmo;
    private JButton volverAEdicionGrafo;
    private JLabel etiqDiferenciaPesos;
    
    //Panel de componentes (panel que contendr las descripciones de las componentes)
    private JPanel panelAristas;
    
    private ArrayList<JTextArea> areasTexto;
    private ArrayList<JLabel> etiquetasFlecha;
    private int aristaActual;
    
    /**
     * Constructor
     * @param panelOpciones panel de opciones disponibles durante la aplicacin de un algoritmo
     * @param areaDibujo rea de dibujo del grafo
     * @param barraMenus barra de mens
     * @param aplicador aplicador del algoritmo
     * @param aristasOrdenadas lista de aristas ordenada por el peso de las mismas
     * @param arbolMasUniforme indica si se va a aplicar el algoritmo de Galil y Schieber
     * para construir el rbol generador ms uniforme
     */
    public GestorPanelOpcionesGalilSchieber(PanelOpciones panelOpciones,
            AreaDibujoGrafo areaDibujo, BarraMenus barraMenus, AplicadorAlgoritmo aplicador,
            Vector<Integer> aristasOrdenadas, boolean arbolMasUniforme) {
        this.aristasOrdenadas = aristasOrdenadas;
        this.arbolMasUniforme = arbolMasUniforme;
        this.panelOpciones = panelOpciones;
        NUM_ARISTAS = aristasOrdenadas.size();
        inicializarNombresAristas();
        JButton elegirArbol, construidoArbol;
        if (arbolMasUniforme) {
            panelNombreAlgoritmo = crearPanelNombreAlgoritmo(
                    "<html><center>Most uniform spanning tree:<br></center>" +
                    "<center>Galil-Schieber algorithm</center></html>");
            elegirArbol = crearBoton(
                    "<html><center>Choose this spanning tree as the<br></center>" +
                    "<center>most uniform one found so far</center></html>", 2);
            construidoArbol = crearBoton(
                    "<html><center>\"Most uniform spanning tree<br></center>" +
                    "<center>already found\"</center></html>", 2);
        }
        else {
            panelNombreAlgoritmo = crearPanelNombreAlgoritmo(
                    "<html><center>Least uniform spanning tree:<br></center>" +
                    "<center>Galil-Schieber algorithm</center></html>");
            elegirArbol = crearBoton(
                    "<html><center>Choose this spanning tree as the<br></center>" +
                    "<center>least uniform one found so far</center></html>", 2);
            construidoArbol = crearBoton(
                    "<html><center>\"Least uniform spanning tree<br></center>" +
                    "<center>already found\"</center></html>", 2);
        }
        elegirArbol.addActionListener(new OyenteElegirArbol(aplicador));
        construidoArbol.addActionListener(new OyenteTerminar(aplicador));
        JLabel etiqElegirArbol = crearEtiqElegirArbol();
        JButton solucionPasoProximo = crearBoton("Solution to next step");
        solucionPasoProximo.addActionListener(new OyenteSolucionPasoProximo(aplicador));
        JButton solucionPasosRestantes = crearBoton("Solution to remaining steps");
        solucionPasosRestantes.addActionListener(new OyenteSolucionPasosRestantes(aplicador));
        volverAEdicionGrafo = crearBoton("Back to graph edition");
        volverAEdicionGrafo.addActionListener(new OyenteVolverAEdicionGrafo(barraMenus,
                panelOpciones, areaDibujo));
        etiqDiferenciaPesos = crearEtiqDiferenciaPesos();
        
        panelOpciones.add(panelNombreAlgoritmo);
        panelOpciones.add(elegirArbol);
        panelOpciones.add(etiqElegirArbol);
        panelOpciones.add(construidoArbol);
        panelOpciones.add(solucionPasoProximo);
        panelOpciones.add(solucionPasosRestantes);
        panelOpciones.add(volverAEdicionGrafo);
        panelOpciones.add(etiqDiferenciaPesos);
        
        if (NUM_ARISTAS > 0) {
            JLabel etiqAristas = crearEtiqAristas();
            crearPanelAristas();
            panelOpciones.add(etiqAristas);
            panelOpciones.setInferiorVisible(true);
        }
        
        panelOpciones.setPreferredSize(new Dimension(260, 447));
        panelOpciones.setVisible(true);
    }
    
    /**
     * Devuelve el array de nombres de las aristas
     * @return el array de nombres de las aristas
     */
    public String[] getNombresAristas() {
        return nombresAristas;
    }
    
    /**
     * Actualiza el panel de opciones
     * @param maxDiferencia la mxima diferencia de pesos entre dos aristas
     * del rbol ms/menos uniforme encontrado hasta el momento
     */
    public void actualizarPanel(int maxDiferencia) {
        
        //Si se recibe como parmetro un valor negativo, en el panel de opciones slo se muestra
        //el nombre del algoritmo y la opcin "Volver a edicin del grafo".
        if (maxDiferencia < 0) {
            panelOpciones.setVisible(false);
            panelOpciones.removeAll();
            panelOpciones.add(panelNombreAlgoritmo);
            panelOpciones.add(volverAEdicionGrafo);
            panelOpciones.setPreferredSize(new Dimension(260, 3000));
            panelOpciones.setVisible(true);
            return;
        }
        
        if (arbolMasUniforme) {
            etiqDiferenciaPesos.setText(
                    "<html>Maximum weight difference of the<br>" +
                    "most uniform spanning tree found<br>" +
                    "so far: " + maxDiferencia + "</html>");
        }
        else {
            etiqDiferenciaPesos.setText(
                    "<html>Maximum weight difference of the<br>" +
                    "least uniform spanning tree found<br>" +
                    "so far: " + maxDiferencia + "</html>");
        }
    }
    
    /**
     * Actualiza el panel de opciones
     *
     */
    public void actualizarPanel() {
        if (NUM_ARISTAS == 0)
            return;
        etiquetasFlecha.get(aristaActual).setIcon(null);
        if (aristaActual < (NUM_ARISTAS - 1)) {
            areasTexto.get(aristaActual + 2).setText(" ");
            etiquetasFlecha.get(aristaActual + 1).setIcon(FLECHA_VERDE);
            aristaActual++;
        }
    }
    
    /**
     * Inicializa la variable nombresAristas (array de nombres de las aristas)
     * 
     */
    private void inicializarNombresAristas() {
        if (NUM_ARISTAS == 0) {
            nombresAristas = null;
            return;
        }
        nombresAristas = new String[NUM_ARISTAS];
        int i = 0;
        for (char car1 = ('a' - 1); car1 < ('z' + 1); car1++)
            for (char car2 = 'a'; car2 < ('z' + 1); car2++) {
                if (i == NUM_ARISTAS)
                    return;
                if (car1 == ('a' - 1))
                    nombresAristas[aristasOrdenadas.get(i).intValue()] = String.valueOf(car2);
                else
                    nombresAristas[aristasOrdenadas.get(i).intValue()] =
                        String.valueOf(car1) + car2;
                i++;
            }
    }
    
    /**
     * Crea un panel en el que se indicar qu algoritmo se aplica
     * @param texto texto a mostrar en el panel
     * @return el panel creado
     */
    private JPanel crearPanelNombreAlgoritmo(String texto) {
        JPanel panel = new JPanel();
        panel.setBackground(Color.BLACK);
        panel.setOpaque(true);
        panel.setPreferredSize(new Dimension(260, 70));
        panel.add(crearEtiqNombreAlgoritmo(texto));
        return panel;
    }
    
    /**
     * Crea una etiqueta en la que se indica qu algoritmo se aplica
     * @param texto el texto de la etiqueta
     * @return la etiqueta creada
     */
    private JLabel crearEtiqNombreAlgoritmo(String texto) {
        JLabel etiqueta = new JLabel(texto);
        etiqueta.setForeground(Color.WHITE);
        etiqueta.setHorizontalAlignment(SwingConstants.CENTER);
        etiqueta.setVerticalAlignment(SwingConstants.CENTER);
        
        //Se cambia el tamao de la fuente
        Font fuente = etiqueta.getFont();
        String nombreFuente = fuente.getName();
        int estiloFuente = fuente.getStyle();
        int tamaoFuente = fuente.getSize() + 2;
        etiqueta.setFont(new Font(nombreFuente, estiloFuente, tamaoFuente));
        
        etiqueta.setPreferredSize(new Dimension(250, 60));
        return etiqueta;
    }
    
    /**
     * Crea un botn con una lnea de texto para el panel de opciones
     * @param texto el texto del botn
     * @return el botn creada
     */
    private JButton crearBoton(String texto) {
        JButton boton = new JButton(texto);
        boton.setBorderPainted(true);
        boton.setPreferredSize(new Dimension(255, 25));
        return boton;
    }
    
    /**
     * Crea un botn para el panel de opciones
     * @param texto el texto del botn
     * @param numLineas el nmero de lneas del texto del botn
     * @return el botn creada
     */
    private JButton crearBoton(String texto, int numLineas) {
        JButton boton = new JButton(texto);
        boton.setBorderPainted(true);
        boton.setPreferredSize(new Dimension(255, 8 + (17 * numLineas)));
        return boton;
    }
    
    /**
     * Crea la etiqueta adjunta al botn elegirArbol
     * @return la etiqueta creada
     */
    private JLabel crearEtiqElegirArbol() {
        JLabel etiqueta = new JLabel(
                "<html>(do not press this button if you think<br>" +
                "any previously found tree is as uniform<br>" +
                "as the tree you have just found)<html>");
        etiqueta.setVerticalAlignment(SwingConstants.TOP);
        etiqueta.setPreferredSize(new Dimension(250, 64));
        
        //Se cambia el estilo de la fuente
        Font fuente = etiqueta.getFont();
        String nombreFuente = fuente.getName();
        int estiloFuente = Font.BOLD;
        int tamaoFuente = fuente.getSize();
        etiqueta.setFont(new Font(nombreFuente, estiloFuente, tamaoFuente));
        
        return etiqueta;
    }
    
    /**
     * Crea la etiqueta en la que se indicar la mxima diferencia de pesos entre dos aristas
     * del rbol ms/menos uniforme encontrado hasta el momento
     * @return la etiqueta creada
     */
    private JLabel crearEtiqDiferenciaPesos() {
        JLabel etiqueta = new JLabel();
        etiqueta.setVerticalAlignment(SwingConstants.BOTTOM);
        etiqueta.setPreferredSize(new Dimension(250, 59));
        
        //Se cambia el estilo de la fuente
        Font fuente = etiqueta.getFont();
        String nombreFuente = fuente.getName();
        int estiloFuente = Font.BOLD;
        int tamaoFuente = fuente.getSize();
        etiqueta.setFont(new Font(nombreFuente, estiloFuente, tamaoFuente));
        
        return etiqueta;
    }
    
    /**
     * Crea una etiqueta para indicar en el panel de opciones que a continuacin se
     * especifican las aristas
     * @return la etiqueta creada
     */
    private JLabel crearEtiqAristas() {
        JLabel etiqueta = new JLabel(
                "<html>Edges (the green arrow indicates which<br>" +
                "is the next edge to be scanned):<html>");
        etiqueta.setVerticalAlignment(SwingConstants.BOTTOM);
        etiqueta.setPreferredSize(new Dimension(250, 43));
        
        //Se cambia el estilo de la fuente
        Font fuente = etiqueta.getFont();
        String nombreFuente = fuente.getName();
        int estiloFuente = Font.BOLD;
        int tamaoFuente = fuente.getSize();
        etiqueta.setFont(new Font(nombreFuente, estiloFuente, tamaoFuente));
        
        return etiqueta;
    }
    
    /**
     * Crea un panel de aristas
     *
     */
    private void crearPanelAristas() {
        panelAristas = panelOpciones.getInterno();
        
        areasTexto = new ArrayList<JTextArea>();
        etiquetasFlecha = new ArrayList<JLabel>();
        aristaActual = 0;
        
        int altoEtiqTexto = 3 + 16;
        JTextArea areaTexto;
        for (int i = 0; i < NUM_ARISTAS; i++) {
            JLabel etiqTexto = new JLabel(nombresAristas[aristasOrdenadas.get(i).intValue()]);
            etiqTexto.setPreferredSize(new Dimension(313, altoEtiqTexto));
            
            //Se cambia el estilo de la fuente
            Font fuente = etiqTexto.getFont();
            String nombreFuente = fuente.getName();
            int estiloFuente = Font.BOLD;
            int tamaoFuente = fuente.getSize();
            etiqTexto.setFont(new Font(nombreFuente, estiloFuente, tamaoFuente));
            
            if (i == 0)
                areaTexto = new JTextArea(" ");
            else
                areaTexto = new JTextArea();
            areaTexto.setBackground(FONDO_OPCIONES);
            areaTexto.setEditable(false);
            areaTexto.setPreferredSize(new Dimension(6, altoEtiqTexto));
            
            JLabel etiqFlecha;
            if (i == 0)
                etiqFlecha = new JLabel(FLECHA_VERDE);
            else
                etiqFlecha = new JLabel();
            etiqFlecha.setHorizontalAlignment(SwingConstants.LEFT);
            etiqFlecha.setPreferredSize(new Dimension(22, 16));
            
            panelAristas.add(areaTexto);
            panelAristas.add(etiqFlecha);
            panelAristas.add(etiqTexto);
            
            areasTexto.add(areaTexto);
            etiquetasFlecha.add(etiqFlecha);
        }
        areaTexto = new JTextArea();
        areaTexto.setBackground(FONDO_OPCIONES);
        areaTexto.setEditable(false);
        areaTexto.setPreferredSize(new Dimension(6, 19));
        panelAristas.add(areaTexto);
        areasTexto.add(areaTexto);
        
        panelAristas.setPreferredSize(
                new Dimension(356, 5 + ((altoEtiqTexto + 5) * (NUM_ARISTAS + 1))));
    }
    
}
