package interfaz;

import javax.swing.*;
import java.awt.*;

import estructuras.*;
import oyentes.*;

/**
 * Clase que implementa la ventana principal de la aplicacin
 * 
 * @author Daniel Guijarro Enriquez
 * @version 1.0
 */
public class VentanaPrincipal extends JFrame {

    private static final long serialVersionUID = 1L;
    private BarraMenus barraMenus;
    private PanelOpciones panelOpciones;
    private JScrollPane panelScrollAreaDibujo;
    private AreaDibujoGrafo areaDibujo;
    
    /**
     * Constructor
     *
     */
    public VentanaPrincipal() {
        super("Spanning trees");
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);  // Salir al cerrar la ventana
        
        //Se obtiene el tamao de la pantalla
        Toolkit tk = Toolkit.getDefaultToolkit();
        Dimension tamanyoPantalla = tk.getScreenSize();
        
        //Se establece el tamao y la posicin de la ventana principal
        int altoVentana = tamanyoPantalla.height;
        int anchoVentana = tamanyoPantalla.width;
        setSize(anchoVentana, altoVentana);
        setLocation(0, 0);
        
        //Se crea el panel JPanel de opciones disponibles durante la aplicacin de un algoritmo
        panelOpciones = new PanelOpciones();
        panelOpciones.setVisible(false);
        
        //Se crea una instancia de las preferencias del usuario por defecto
        Preferencias preferencias = new Preferencias();
        
        //Se crea el rea de dibujo
        areaDibujo = new AreaDibujoGrafo(preferencias);
        
        //Se crea la barra de mens
        barraMenus = new BarraMenus();
        
        //Se crea un objeto oyente de la clase OyenteRaton y se registra en el rea de dibujo
        OyenteRaton oyenteRaton = new OyenteRaton(barraMenus, areaDibujo);
        areaDibujo.addMouseListener(oyenteRaton);
        areaDibujo.addMouseMotionListener(oyenteRaton);
        
        //Se crea un objeto oyente de la clase OyenteTeclado y se registra en el rea de dibujo
        areaDibujo.addKeyListener(new OyenteTeclado(barraMenus, areaDibujo));
        
        //Se registran los oyentes que corresponda en los
        //objetos que componen la barra de mens y se incluye sta en la ventana
        barraMenus.registrarOyentes(panelOpciones, areaDibujo, preferencias,
                this, oyenteRaton);
        setJMenuBar(barraMenus);
        
        //Se aade el panel de opciones a la ventana
        getContentPane().add(panelOpciones, BorderLayout.WEST);
        
        //Se crea otro panel JScrollPane que contendr el rea de dibujo
        panelScrollAreaDibujo = new JScrollPane(areaDibujo);
        
        //Se aade el panel a la ventana
        getContentPane().add(panelScrollAreaDibujo, BorderLayout.CENTER);
        
        setVisible(true);
    }
    
}
