package oyentes;

import interfaz.*;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.StringReader;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import estructuras.OperacionesFicheros;

/**
 * Clase oyente del evento generado al elegir la opcin "Abrir"
 * 
 * @author Daniel Guijarro Enriquez
 * @version 1.0
 */
public class OyenteAbrir implements ActionListener {

    private BarraMenus barraMenus;
    private AreaDibujoGrafo areaDibujo;
    
    /**
     * Constructor
     * @param barra barra de mens de la aplicacin
     * @param area rea en la que se permitir dibujar un grafo
     */
    public OyenteAbrir(BarraMenus barra, AreaDibujoGrafo area) {
        super();
        barraMenus = barra;
        areaDibujo = area;
    }
    
    /**
     * Mtodo que se invoca cuando ocurre una accin
     */
    public void actionPerformed(ActionEvent e) {
        JFileChooser dlgAbrir = new EnglishFileChooser();
        int opcion = dlgAbrir.showOpenDialog(areaDibujo.getParent());
        if (opcion == JFileChooser.APPROVE_OPTION) {
            String nombreFichero = dlgAbrir.getSelectedFile().getPath();
            Object[] opciones = {"OK"};
            try {
                String contenidoFichero = OperacionesFicheros.leer(nombreFichero);
                BufferedReader br = new BufferedReader(new StringReader(contenidoFichero));
                String linea = br.readLine();
                br.close();
                if (linea.compareTo("fichero datos grafo") != 0) {
                    JOptionPane.showOptionDialog(null,
                            "File \"" + nombreFichero + "\" is not valid",
                            "Error", JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE,
                            null, opciones, null);
                    return;
                }
                areaDibujo.reconstruirGrafo(contenidoFichero);
                barraMenus.desactivarOpcionesNodoYArista();
            } catch (Exception e1) {
                JOptionPane.showOptionDialog(null,
                        "Cannot open \"" + nombreFichero + "\"",
                        "Error", JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE,
                        null, opciones, null);
            }
        }
    }
    
}
