package oyentes;

import interfaz.*;

import javax.swing.event.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;

import javax.swing.*;

import estructuras.OperacionesFicheros;

/**
 * Clase oyente del evento generado al elegir la opcin "Change weight" (Cambiar peso)
 * 
 * @author Daniel Guijarro Enriquez
 * @version 1.0
 */
public class OyenteCrearGrafoPredisenyado implements ActionListener {

    private BarraMenus barraMenus;
    private AreaDibujoGrafo areaDibujo;
    private JDialog ventana;
    
    /**
     * Constructor
     * @param barra barra de mens de la aplicacin
     * @param area rea en la que se permitir dibujar un grafo
     * @param ventanaPrincipal ventana principal de la aplicacin
     */
    public OyenteCrearGrafoPredisenyado(BarraMenus barra, AreaDibujoGrafo area,
            VentanaPrincipal ventanaPrincipal) {
        super();
        barraMenus = barra;
        areaDibujo = area;
        
        ventana = new JDialog(ventanaPrincipal, "Create predesigned graph", true);
        ventana.setVisible(false);
        
        establecerTamanyoYPosicion();
        
        //Se crea una etiqueta, un botn y un panel con barras de desplazamiento
        JLabel etiqueta = new JLabel("Click on the graph you want to create");
        JButton boton = new JButton("Cancel");
        JScrollPane panelScroll = crearPanelScroll();
        
        //Se registra en el botn el oyente correspondiente
        boton.addActionListener(new OyenteCancelar());
        
        //Se incluyen los componentes creados en la ventana
        JPanel panelSuperior = new JPanel(new FlowLayout(FlowLayout.LEFT));
        panelSuperior.setPreferredSize(new Dimension(30, 30));
        panelSuperior.add(etiqueta);
        JPanel panelInferiorDerecho = new JPanel();
        panelInferiorDerecho.setPreferredSize(new Dimension(123, 123));
        panelInferiorDerecho.add(boton);
        JPanel panelInferior = new JPanel(new BorderLayout());
        panelInferior.add(panelInferiorDerecho, BorderLayout.EAST);
        panelInferior.add(panelScroll, BorderLayout.CENTER);
        ventana.getContentPane().add(panelSuperior, BorderLayout.NORTH);
        ventana.getContentPane().add(panelInferior, BorderLayout.CENTER);
    }
    
    /**
     * Mtodo que se invoca cuando ocurre una accin
     */
    public void actionPerformed(ActionEvent e) {
        ventana.setVisible(true);
    }
    
    /**
     * Establece el tamao y la posicin de la ventana
     *
     */
    private void establecerTamanyoYPosicion() {
        
        //Se obtiene el tamao de la pantalla
        Toolkit tk = Toolkit.getDefaultToolkit();
        Dimension tamanyoPantalla = tk.getScreenSize();
        
        //Se establece el tamao y la posicin de la ventana
        if (tamanyoPantalla.width < 1024) {
            ventana.setSize(tamanyoPantalla.width, tamanyoPantalla.height);
            ventana.setLocation(0, 0);
        }
        else {
            int anchoVentana = 920;
            ventana.setSize(anchoVentana, tamanyoPantalla.height);
            ventana.setLocation((tamanyoPantalla.width - anchoVentana) / 2, 0);
        }
    }
    
    /**
     * Crea el panel con barras de desplazamiento que se incluir en la ventana que
     * permitir al usuario elegir un grafo prediseado. El panel creado ya incluye los
     * componentes correspondientes
     * @return el panel creado
     */
    private JScrollPane crearPanelScroll() {
        JPanel panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        panel.setPreferredSize(new Dimension(770, 2030));
        
        OyenteElegirGrafo oyente = new OyenteElegirGrafo();
        
        boolean leidos = false;
        int numFichero = 0;
        String contenidoFichero;
        while (!leidos) {
            numFichero++;
            try {
                contenidoFichero = OperacionesFicheros.leer(
                        this.getClass().getResource("/ejemplos/grafo" + numFichero));
            } catch (Exception e) {
                leidos = true;
                continue;
            }
            MinipanelGrafo minipanel = new MinipanelGrafo(contenidoFichero);
            minipanel.addMouseListener(oyente);
            JLabel etiqueta = null;
            switch (numFichero) {
            case 1:
                etiqueta = new JLabel("Petersen");
                break;
            case 2:
                etiqueta = new JLabel("Desargues");
                break;
            case 3:
                etiqueta = new JLabel("Heawood");
                break;
            case 4:
                etiqueta = new JLabel("Franklin");
                break;
            case 5:
                etiqueta = new JLabel("Tutte");
                break;
            case 6:
                etiqueta = new JLabel("Herschel");
                break;
            case 7:
                etiqueta = new JLabel("Tetrahedron");
                break;
            case 8:
                etiqueta = new JLabel("Octahedron");
                break;
            case 9:
                etiqueta = new JLabel("Cube");
                break;
            case 10:
                etiqueta = new JLabel("Dodecahedron");
                break;
            case 11:
                etiqueta = new JLabel("Icosahedron");
                break;
            default:
                etiqueta = new JLabel();
                break;
            }
            etiqueta.setPreferredSize(new Dimension(240, 20));
            etiqueta.setHorizontalAlignment(JLabel.CENTER);
            JPanel subpanel = new JPanel();
            subpanel.setPreferredSize(new Dimension(250, 220));
            subpanel.add(minipanel);
            subpanel.add(etiqueta);
            panel.add(subpanel);
        }
        
        return new JScrollPane(panel);
    }
    
    /**
     * Clase oyente del evento producido al pulsar el boton "Cancel"
     */
    class OyenteCancelar implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            ventana.setVisible(false);
        }
    }
    
    /**
     * Clase oyente del evento producido al elegir un grafo
     */
    class OyenteElegirGrafo extends MouseInputAdapter {
        public void mouseClicked(MouseEvent e) {
            
            //Se obtiene el minipanel fuente del evento
            MinipanelGrafo minipanel = (MinipanelGrafo)e.getSource();
            
            areaDibujo.reconstruirGrafo(minipanel.cadenaDeGrafo());
            barraMenus.desactivarOpcionesNodoYArista();
            ventana.setVisible(false);
        }
    }
    
}
