package oyentes;

import interfaz.*;
import estructuras.*;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.*;

/**
 * Clase oyente del evento generado al elegir la opcin "Camino de capacidad mxima"
 * 
 * @author Daniel Guijarro Enriquez
 * @version 1.0
 */
public class OyenteElegirCaminoCapacidadMaxima implements ActionListener {

    private BarraMenus barraMenus;
    private PanelOpciones panelOpciones;
    private AreaDibujoGrafo areaDibujo;
    private OyenteRaton oyenteRaton;
    
    /**
     * Constructor
     * @param barra barra de mens
     * @param panelOpciones panel de opciones disponibles durante la aplicacin de un algoritmo
     * @param area rea en la que se permitir dibujar un grafo
     * @param oyenteRaton objeto oyente de eventos generados al realizar una accin con el ratn
     */
    public OyenteElegirCaminoCapacidadMaxima(BarraMenus barra, PanelOpciones panelOpciones,
            AreaDibujoGrafo area, OyenteRaton oyenteRaton) {
        super();
        barraMenus = barra;
        this.panelOpciones = panelOpciones;
        areaDibujo = area;
        this.oyenteRaton = oyenteRaton;
    }
    
    /**
     * Mtodo que se invoca cuando ocurre una accin
     */
    public void actionPerformed(ActionEvent e) {
        Object[] opciones = {"OK"};
        if (areaDibujo.grafoEsEditable()) {
            if (!areaDibujo.grafoEsConexo()) {
                JOptionPane.showOptionDialog(null, "A connected graph must be created first",
                        "Error", JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE,
                        null, opciones, null);
                return;
            }
            if (areaDibujo.numNodosGrafo() < 2) {
                JOptionPane.showOptionDialog(null, "Graph not accepted: it has less than " +
                        "2 vertices", "Error", JOptionPane.DEFAULT_OPTION,
                        JOptionPane.ERROR_MESSAGE, null, opciones, null);
                return;
            }
            areaDibujo.setGrafoEditable(false);
            barraMenus.activarSoloOpcionesAlgoritmo();
            AplicadorCaminoCapacidadMaxima aplicador = new AplicadorCaminoCapacidadMaxima(
                    areaDibujo, panelOpciones, barraMenus);
            oyenteRaton.setAplicadorAlgoritmo(aplicador);
        }
        else {
            if (areaDibujo.numNodosGrafo() < 2) {
                JOptionPane.showOptionDialog(null, "Graph not accepted: it has less than " +
                        "2 vertices", "Error", JOptionPane.DEFAULT_OPTION,
                        JOptionPane.ERROR_MESSAGE, null, opciones, null);
                return;
            }
            panelOpciones.setVisible(false);
            areaDibujo.limpiarGrafo();
            panelOpciones.removeAll();
            AplicadorCaminoCapacidadMaxima aplicador = new AplicadorCaminoCapacidadMaxima(
                    areaDibujo, panelOpciones, barraMenus);
            oyenteRaton.setAplicadorAlgoritmo(aplicador);
        }
    }
    
}
