package oyentes;

import interfaz.*;
import estructuras.*;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.*;

/**
 * Clase oyente del evento generado al elegir la opcin
 * "Arbol generador mnimo" -> "Algoritmo de Kruskal"
 * 
 * @author Daniel Guijarro Enriquez
 * @version 1.0
 */
public class OyenteElegirKruskalMinimo implements ActionListener {

    private BarraMenus barraMenus;
    private PanelOpciones panelOpciones;
    private AreaDibujoGrafo areaDibujo;
    private OyenteRaton oyenteRaton;
    
    /**
     * Constructor
     * @param barra barra de mens
     * @param panelOpciones panel de opciones disponibles durante la aplicacin de un algoritmo
     * @param area rea en la que se permitir dibujar un grafo
     * @param oyenteRaton objeto oyente de eventos generados al realizar una accin con el ratn
     */
    public OyenteElegirKruskalMinimo(BarraMenus barra, PanelOpciones panelOpciones,
            AreaDibujoGrafo area, OyenteRaton oyenteRaton) {
        super();
        barraMenus = barra;
        this.panelOpciones = panelOpciones;
        areaDibujo = area;
        this.oyenteRaton = oyenteRaton;
    }
    
    /**
     * Mtodo que se invoca cuando ocurre una accin
     */
    public void actionPerformed(ActionEvent e) {
        Object[] opciones = {"OK"};
        if (areaDibujo.grafoEsEditable()) {
            if (!areaDibujo.grafoEsConexo()) {
                JOptionPane.showOptionDialog(null, "A connected graph must be created first",
                        "Error", JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE,
                        null, opciones, null);
                return;
            }
            areaDibujo.setGrafoEditable(false);
            barraMenus.activarSoloOpcionesAlgoritmo();
            AplicadorKruskalMinimo aplicador = new AplicadorKruskalMinimo(areaDibujo,
                    panelOpciones);
            oyenteRaton.setAplicadorAlgoritmo(aplicador);
            aplicador.prepararAreaDibujo();
            aplicador.prepararPanelOpcionesPrincipio(barraMenus);
        }
        else {
            panelOpciones.setVisible(false);
            areaDibujo.limpiarGrafo();
            panelOpciones.removeAll();
            AplicadorKruskalMinimo aplicador = new AplicadorKruskalMinimo(areaDibujo,
                    panelOpciones);
            oyenteRaton.setAplicadorAlgoritmo(aplicador);
            aplicador.prepararAreaDibujo();
            aplicador.prepararPanelOpcionesPrincipio(barraMenus);
        }
    }
    
}
