package oyentes;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;

import estructuras.*;

import interfaz.*;

/**
 * Clase oyente del evento generado al elegir la opcin
 * "Arbol generador ms uniforme"
 * 
 * @author Daniel Guijarro Enriquez
 * @version 1.0
 */
public class OyenteElegirMasUniforme implements ActionListener {

    private BarraMenus barraMenus;
    private PanelOpciones panelOpciones;
    private AreaDibujoGrafo areaDibujo;
    private OyenteRaton oyenteRaton;
    
    /**
     * Constructor
     * @param barra barra de mens
     * @param panelOpciones panel de opciones disponibles durante la aplicacin de un algoritmo
     * @param area rea en la que se permitir dibujar un grafo
     * @param oyenteRaton objeto oyente de eventos generados al realizar una accin con el ratn
     */
    public OyenteElegirMasUniforme(BarraMenus barra, PanelOpciones panelOpciones,
            AreaDibujoGrafo area, OyenteRaton oyenteRaton) {
        super();
        barraMenus = barra;
        this.panelOpciones = panelOpciones;
        areaDibujo = area;
        this.oyenteRaton = oyenteRaton;
    }
    
    /**
     * Mtodo que se invoca cuando ocurre una accin
     */
    public void actionPerformed(ActionEvent e) {
        Object[] opciones = {"OK"};
        int algoritmo = VentanaMasUniforme.mostrarVentana();
        switch (algoritmo) {
        case VentanaMasUniforme.GALIL_SCHIEBER:
            if (areaDibujo.grafoEsEditable()) {
                if (!areaDibujo.grafoEsConexo()) {
                    JOptionPane.showOptionDialog(null, "A connected graph must be created first",
                            "Error", JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE,
                            null, opciones, null);
                    return;
                }
                if (areaDibujo.numAristasGrafo() > 702) {
                    JOptionPane.showOptionDialog(null, "Graph not accepted: it has more than " +
                            "702 edges", "Error", JOptionPane.DEFAULT_OPTION,
                            JOptionPane.ERROR_MESSAGE, null, opciones, null);
                    return;
                }
                areaDibujo.setGrafoEditable(false);
                barraMenus.activarSoloOpcionesAlgoritmo();
                AplicadorGalilSchieberMasUniforme aplicador =
                    new AplicadorGalilSchieberMasUniforme(areaDibujo, panelOpciones, barraMenus);
                oyenteRaton.setAplicadorAlgoritmo(aplicador);
            }
            else {
                if (areaDibujo.numAristasGrafo() > 702) {
                    JOptionPane.showOptionDialog(null, "Graph not accepted: it has more than " +
                            "702 edges", "Error", JOptionPane.DEFAULT_OPTION,
                            JOptionPane.ERROR_MESSAGE, null, opciones, null);
                    return;
                }
                panelOpciones.setVisible(false);
                areaDibujo.limpiarGrafo();
                panelOpciones.removeAll();
                AplicadorGalilSchieberMasUniforme aplicador =
                    new AplicadorGalilSchieberMasUniforme(areaDibujo, panelOpciones, barraMenus);
                oyenteRaton.setAplicadorAlgoritmo(aplicador);
            }
            break;
        case VentanaMasUniforme.A:
            if (areaDibujo.grafoEsEditable()) {
                if (!areaDibujo.grafoEsConexo()) {
                    JOptionPane.showOptionDialog(null, "A connected graph must be created first",
                            "Error", JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE,
                            null, opciones, null);
                    return;
                }
                if (areaDibujo.sumaPesosGrafo() > Integer.MAX_VALUE) {
                    JOptionPane.showOptionDialog(null, "Graph not accepted: the sum of the " +
                            "weights is greater than " + Integer.MAX_VALUE,
                            "Error", JOptionPane.DEFAULT_OPTION,
                            JOptionPane.ERROR_MESSAGE, null, opciones, null);
                    return;
                }
                areaDibujo.setGrafoEditable(false);
                barraMenus.activarSoloOpcionesAlgoritmo();
                AplicadorAMasUniforme aplicador = new AplicadorAMasUniforme(areaDibujo,
                        panelOpciones);
                oyenteRaton.setAplicadorAlgoritmo(aplicador);
                aplicador.prepararAreaDibujo();
                aplicador.prepararPanelOpcionesPrincipio(barraMenus);
            }
            else {
                if (areaDibujo.sumaPesosGrafo() > Integer.MAX_VALUE) {
                    JOptionPane.showOptionDialog(null, "Graph not accepted: the sum of the " +
                            "weights is greater than " + Integer.MAX_VALUE,
                            "Error", JOptionPane.DEFAULT_OPTION,
                            JOptionPane.ERROR_MESSAGE, null, opciones, null);
                    return;
                }
                panelOpciones.setVisible(false);
                areaDibujo.limpiarGrafo();
                panelOpciones.removeAll();
                AplicadorAMasUniforme aplicador = new AplicadorAMasUniforme(areaDibujo,
                        panelOpciones);
                oyenteRaton.setAplicadorAlgoritmo(aplicador);
                aplicador.prepararAreaDibujo();
                aplicador.prepararPanelOpcionesPrincipio(barraMenus);
            }
            break;
        }
    }
    
}
