package oyentes;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.*;

import estructuras.*;
import interfaz.*;

/**
 * Clase oyente del evento generado al elegir la opcin "Weights for new edges"
 * (Pesos para nuevas aristas)
 * 
 * @author Daniel Guijarro Enriquez
 * @version 1.0
 */
public class OyentePesosParaNuevasAristas implements ActionListener {

    private Preferencias preferencias;
    private JDialog ventana;
    private JRadioButton opcionPesoNoAleatorio;
    private JRadioButton opcionPesoAleatorio;
    private JTextField campoTextoExactamente;
    private JTextField campoTextoDesde;
    private JTextField campoTextoHasta;
    
    /**
     * Constructor
     * @param preferencias preferencias del usuario
     * @param ventanaPrincipal ventana principal de la aplicacin
     */
    public OyentePesosParaNuevasAristas(Preferencias preferencias,
            VentanaPrincipal ventanaPrincipal) {
        
        this.preferencias = preferencias;
        
        ventana = new JDialog(ventanaPrincipal, "Weights for new edges", true);
        ventana.setVisible(false);
        
        //Se obtiene el tamao de la pantalla
        Toolkit tk = Toolkit.getDefaultToolkit();
        Dimension tamanyoPantalla = tk.getScreenSize();
        
        //Se establece el tamao y la posicin de la ventana
        int anchoVentana = 640;
        int altoVentana = 160;
        ventana.setSize(anchoVentana, altoVentana);
        ventana.setLocation((tamanyoPantalla.width - anchoVentana) / 2,
                (tamanyoPantalla.height - altoVentana) / 2);
        
        //Crea un grupo de botones de radio y se incluyen en el mismo dos botones de radio
        ButtonGroup grupo = new ButtonGroup();
        opcionPesoNoAleatorio = new JRadioButton("Create edges with a fixed weight: ", true);
        opcionPesoAleatorio = new JRadioButton("Create edges with a random weight " +
                "from");
        grupo.add(opcionPesoNoAleatorio);
        grupo.add(opcionPesoAleatorio);
        
        //Se crean otros componentes (campos de texto, etiquetas y botones)
        campoTextoExactamente = new JTextField(7); //Campo de texto de 7 columnas
        campoTextoDesde = new JTextField(7);
        JLabel etiqHasta = new JLabel(" to ");
        campoTextoHasta = new JTextField(7);
        JLabel etiqAmbosIncluidos = new JLabel(" inclusive");
        JButton botonAceptar = new JButton("OK");
        JButton botonCancelar = new JButton("Cancel");
        
        //Se crea el oyente para los botones "OK" y "Cancel" y se registra en
        //dichos botones
        OyenteAccion oyente = new OyenteAccion();
        botonAceptar.addActionListener(oyente);
        botonCancelar.addActionListener(oyente);
        
        //Se crean los paneles que incluirn los campos de texto, etiquetas y botones
        JPanel panelNoAleatorio = new JPanel();
        panelNoAleatorio.setLayout(new FlowLayout(FlowLayout.LEFT));
        panelNoAleatorio.setPreferredSize(new Dimension(640, 30));
        JPanel panelAleatorio = new JPanel();
        panelAleatorio.setLayout(new FlowLayout(FlowLayout.LEFT));
        panelAleatorio.setPreferredSize(new Dimension(640, 30));
        JPanel panelBotones = new JPanel();
        panelAleatorio.setPreferredSize(new Dimension(640, 40));
        
        //Se aaden a los paneles los componentes que corresponda
        panelNoAleatorio.add(opcionPesoNoAleatorio);
        panelNoAleatorio.add(campoTextoExactamente);
        panelAleatorio.add(opcionPesoAleatorio);
        panelAleatorio.add(campoTextoDesde);
        panelAleatorio.add(etiqHasta);
        panelAleatorio.add(campoTextoHasta);
        panelAleatorio.add(etiqAmbosIncluidos);
        panelBotones.add(botonAceptar);
        panelBotones.add(botonCancelar);
        
        //Se aaden los paneles a la ventana
        ventana.getContentPane().setLayout(new FlowLayout());
        ventana.getContentPane().add(panelNoAleatorio);
        ventana.getContentPane().add(panelAleatorio);
        ventana.getContentPane().add(panelBotones);
    }
    
    /**
     * Mtodo que se invoca cuando ocurre una accin
     */
    public void actionPerformed(ActionEvent e) {
        if (preferencias.sonPesosAleatorios()) {
            opcionPesoNoAleatorio.setSelected(false);
            opcionPesoAleatorio.setSelected(true);
        }
        else {
            opcionPesoNoAleatorio.setSelected(true);
            opcionPesoAleatorio.setSelected(false);
        }
        campoTextoExactamente.setText(String.valueOf(preferencias.getPesoNoAleatorio()));
        campoTextoDesde.setText(String.valueOf(preferencias.getPesoMinimo()));
        campoTextoHasta.setText(String.valueOf(preferencias.getPesoMaximo()));
        ventana.setVisible(true);
    }
    
    /**
     * Clase oyente de eventos de accin
     * 
     * @author Daniel Guijarro Enriquez
     * @version 1.0
     */
    class OyenteAccion implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Object[] opciones = {"OK"};
            if (((JButton)e.getSource()).getText().compareTo("OK") == 0) {
                if (opcionPesoAleatorio.isSelected()) {
                    int pesoMinimo, pesoMaximo;
                    
                    //Se comprueba si el peso mnimo es vlido
                    try {
                        pesoMinimo = Integer.parseInt(campoTextoDesde.getText());
                    } catch (NumberFormatException e1) {
                        JOptionPane.showOptionDialog(null, "Invalid minimum weight", "Error",
                                JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE,
                                null, opciones, null);
                        return;
                    }
                    if ((pesoMinimo <= 0) || (pesoMinimo == Integer.MAX_VALUE)) {
                        JOptionPane.showOptionDialog(null, "Invalid minimum weight", "Error",
                                JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE,
                                null, opciones, null);
                        return;
                    }
                    
                    //Se comprueba si el peso mximo es vlido
                    try {
                        pesoMaximo = Integer.parseInt(campoTextoHasta.getText());
                    } catch (NumberFormatException e1) {
                        JOptionPane.showOptionDialog(null, "Invalid maximum weight", "Error",
                                JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE,
                                null, opciones, null);
                        return;
                    }
                    if ((pesoMaximo <= 0) || (pesoMaximo == Integer.MAX_VALUE)) {
                        JOptionPane.showOptionDialog(null, "Invalid maximum weight", "Error",
                                JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE,
                                null, opciones, null);
                        return;
                    }
                    
                    //Se comprueba si el peso mximo es mayor o igual que el mnimo
                    if (pesoMaximo < pesoMinimo) {
                        JOptionPane.showOptionDialog(null, "The maximum weight cannot be " +
                                "lesser than the minimum weight", "Error",
                                JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE,
                                null, opciones, null);
                        return;
                    }
                    
                    //Se establece que los pesos que se generen para nuevas aristas deben ser
                    //aleatorios y comprendidos entre el peso mximo y el mnimo
                    preferencias.setPesoAleatorio(pesoMinimo, pesoMaximo);
                }
                else {  //Peso no aleatorio
                    int peso;
                    
                    //Se comprueba si el peso es vlido
                    try {
                        peso = Integer.parseInt(campoTextoExactamente.getText());
                    } catch (NumberFormatException e1) {
                        JOptionPane.showOptionDialog(null, "Invalid weight", "Error",
                                JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE,
                                null, opciones, null);
                        return;
                    }
                    if ((peso <= 0) || (peso == Integer.MAX_VALUE)) {
                        JOptionPane.showOptionDialog(null, "Invalid weight", "Error",
                                JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE,
                                null, opciones, null);
                        return;
                    }
                    
                    //Se establece que se genere el peso introducido para nuevas aristas
                    preferencias.setPesoNoAleatorio(peso);
                }
            }
            ventana.setVisible(false);
        }
    }
    
}
