package oyentes;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.*;

import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.event.*;

import interfaz.*;
import estructuras.*;

/**
 * Clase oyente de eventos generados al realizar una accin con el ratn
 * 
 * @author Daniel Guijarro Enriquez
 * @version 1.0
 */
public class OyenteRaton extends MouseInputAdapter {

    private BarraMenus barraMenus;
    private AreaDibujoGrafo areaDibujo;
    private AplicadorAlgoritmo aplicadorAlgoritmo;
    
    /**
     * Constructor
     * @param barra barra de mens de la aplicacin
     * @param area rea en la que se permitir dibujar un grafo
     */
    public OyenteRaton(BarraMenus barra, AreaDibujoGrafo area) {
        super();
        barraMenus = barra;
        areaDibujo = area;
    }
    
    /**
     * Mtodo que se invoca cuando se hace clic con el ratn
     */
    public void mouseClicked(MouseEvent e) {
        areaDibujo.requestFocusInWindow();
        int x = e.getX();
        int y = e.getY();
        if (areaDibujo.grafoEsEditable()) {
            boolean seleccionadoNodo = false;
            boolean seleccionadaArista = false;
            IdElemento idElemPulsado = areaDibujo.localizarElemento(x, y);
            if (idElemPulsado.tipo == IdElemento.NODO) {
                areaDibujo.deseleccionarAristasSeleccionadas();
                IdElemento idElemSeleccionado = areaDibujo.localizarNodoSeleccionado();
                if (idElemSeleccionado.tipo == IdElemento.VACIO) {
                    areaDibujo.seleccionarNodo(idElemPulsado.id);
                    seleccionadoNodo = true;
                }
                else {
                    if (idElemSeleccionado.id == idElemPulsado.id)
                        areaDibujo.deseleccionarNodosSeleccionados();
                    else
                        areaDibujo.crearArista(idElemSeleccionado.id, idElemPulsado.id);
                }
            }
            else if (idElemPulsado.tipo == IdElemento.ARISTA) {
                areaDibujo.deseleccionarNodosSeleccionados();
                if (e.getClickCount() > 1) {  // Si se ha hecho doble clic...
                    areaDibujo.permitirCambiarPesoArista(idElemPulsado.id);
                    areaDibujo.deseleccionarAristasSeleccionadas();
                }
                else {  // Si se ha hecho clic simple...
                    IdElemento idElemSeleccionado = areaDibujo.localizarAristaSeleccionada();
                    if (idElemSeleccionado.tipo == IdElemento.VACIO) {
                        areaDibujo.seleccionarArista(idElemPulsado.id);
                        seleccionadaArista = true;
                    }
                    else {
                        areaDibujo.deseleccionarAristasSeleccionadas();
                        if (idElemSeleccionado.id != idElemPulsado.id) {
                            areaDibujo.seleccionarArista(idElemPulsado.id);
                            seleccionadaArista = true;
                        }
                    }
                }
            }
            else {  // Si se ha hecho clic sobre vaco...
                areaDibujo.deseleccionarElementosSeleccionados();
                areaDibujo.crearNodo(x, y);
            }
            if (seleccionadaArista)
                barraMenus.activarOpcionesArista();
            else if (seleccionadoNodo)
                barraMenus.activarOpcionesNodo();
            else
                barraMenus.desactivarOpcionesNodoYArista();
        }
        else {
            IdElemento idElemPulsado = areaDibujo.localizarElemento(x, y);
            boolean pasoCorrecto = true;
            if (idElemPulsado.tipo == IdElemento.NODO)
                pasoCorrecto = aplicadorAlgoritmo.elegirVertice(idElemPulsado.id);
            else if (idElemPulsado.tipo == IdElemento.ARISTA)
                pasoCorrecto = aplicadorAlgoritmo.elegirArista(idElemPulsado.id);
            if (!pasoCorrecto) {
                Object[] opciones = {"OK"};
                JOptionPane panel = new JOptionPane("The step you have taken is WRONG",
                        JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION, null, opciones);
                JDialog ventana = panel.createDialog(null, "WRONG");
                
                //Se obtiene el tamao de la pantalla
                Toolkit tk = Toolkit.getDefaultToolkit();
                Dimension tamanyoPantalla = tk.getScreenSize();
                
                //Se establece el tamao y la posicin de la ventana
                ventana.setLocation((tamanyoPantalla.width - ventana.getPreferredSize().width) / 2,
                        tamanyoPantalla.height - ((ventana.getPreferredSize().height * 6) / 5));
                
                ventana.setModal(true);
                ventana.setVisible(true);
            }
        }
    }
    
    /**
     * Mtodo que se invoca cuando se pulsa un botn del ratn y, mantenindolo pulsado,
     * se mueve el ratn
     */
    public void mouseDragged(MouseEvent e) {
        if (areaDibujo.grafoEsEditable()) {
            int x = e.getX();
            int y = e.getY();
            IdElemento idElemento = areaDibujo.localizarNodo(x, y);
            if (idElemento.tipo == IdElemento.NODO)
                areaDibujo.moverNodo(idElemento.id, x, y);
        }
    }
    
    /**
     * Establece el aplicador de algoritmo
     * @param aplicador el aplicador de algoritmo a establecer
     */
    public void setAplicadorAlgoritmo(AplicadorAlgoritmo aplicador) {
        aplicadorAlgoritmo = aplicador;
    }
    
}
